﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using KMCCC.Authentication;
using KMCCC.Launcher;
using KMCCC.Tools;
using KMCCC.Modules;
using KMCCC.Modules.JVersion;
using CityCraft;
using System.IO;
using System.Windows.Threading;
using System.Windows.Media.Animation;
using LitJson;
using CTZLauncher.Modules.CTZServer;

namespace CTZLauncher
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        LauncherCore launcher = null;
        LaunchOptions option = new LaunchOptions();
        string serveraddress = "CityCraft.cn";
        int serverport = 25580;

        #region 初始化部分
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            if (!Directory.Exists(".minecraft"))
            {
                Directory.CreateDirectory(".minecraft");
            }
            launcher = LauncherCore.Create(".minecraft");
        }
        #endregion

        #region 登录界面
        private void barclick_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Label bar = (Label)sender;
            MessageBox.Show(bar.Name);
            switch (bar.Name.Substring(4))
            {
                case "l1":
                    break;
                case "l2":
                    break;
                case "l3":
                    break;
                case "l4":
                    break;
                case "r1":
                    break;
                case "r2":
                    break;
                case "r3":
                    break;
                case "r4":
                    break;
            }
        }

        private void Login_Click(object sender, RoutedEventArgs e)
        {
            if (username.Text.Length == 0 || password.Text.Length == 0)
            {
                MessageBox.Show("请输入账号密码!");
                return;
            }
            Login.IsEnabled = false;
            CTZAuthenticator auth = new CTZAuthenticator(username.Text, password.Text, serveraddress, serverport);
            if (auth.isLogin())
            {
                MessageBox.Show("当前玩家已登录服务器!");
            }
            else
            {
                if (!auth.Login())
                {
                    if (!auth.isRegistered())
                        MessageBox.Show("该用户名未注册 请先注册!");
                    else
                        MessageBox.Show("登录失败 账号不存在 或 密码错误 !");
                }
                else
                {
                    ServerWindow.Visibility = System.Windows.Visibility.Visible;
                    RotateTransform rtf = new RotateTransform();
                    ServerWindow.RenderTransform = rtf;
                    LoginWindow.RenderTransform = rtf;
                    DoubleAnimation dbAscending = new DoubleAnimation(0, 360, new Duration(TimeSpan.FromSeconds(0.75)));
                    dbAscending.RepeatBehavior = new RepeatBehavior(1);
                    rtf.BeginAnimation(RotateTransform.AngleProperty, dbAscending);
                    CTZServer areas = JsonMapper.ToObject<CTZServer>(auth.getServerList().ToLower());
                    LoadAreas(areas);
                }
            }
            Login.IsEnabled = true;
        }

        private void outline_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
                this.DragMove();
        }

        private void close_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void maxmem_TextChanged(object sender, TextChangedEventArgs e)
        {
            //屏蔽中文输入和非法字符粘贴输入
            TextBox textBox = sender as TextBox;
            TextChange[] change = new TextChange[e.Changes.Count];
            e.Changes.CopyTo(change, 0);

            int offset = change[0].Offset;
            if (change[0].AddedLength > 0)
            {
                double num = 0;
                if (!Double.TryParse(textBox.Text, out num))
                {
                    textBox.Text = textBox.Text.Remove(offset, change[0].AddedLength);
                    textBox.Select(offset, 0);
                }
            }
        }

        private void register_Click(object sender, RoutedEventArgs e)
        {
            if (username.Text.Length == 0 || password.Text.Length == 0)
            {
                MessageBox.Show("请输入账号密码!");
                return;
            }
            CTZAuthenticator auth = new CTZAuthenticator(username.Text, password.Text, "citycraft.cn", serverport);
            if (auth.isRegistered())
            {
                MessageBox.Show("该用户名已注册 请更换用户名!");
                return;
            }
            if (auth.Register())
            {
                MessageBox.Show("注册成功!");
                auth.Login();
                ServerWindow.Visibility = System.Windows.Visibility.Visible;
            }
            else
                MessageBox.Show("注册失败!");
        }
        #endregion

        #region 服务器选择与启动
        private void LoadAreas(CTZServer areas)
        {
            for (int i = 0; i < areas.Areas.Count; i++)
            {
                Area area = areas.Areas[i];
                RadioButton arearb = GameArea.FindName("a" + i) as RadioButton;
                arearb.Content = area.Name;
                arearb.Visibility = Visibility.Visible;
                arearb.Tag = area.Servers;
            }
        }

        private void LoadServers(List<Server> servers)
        {
            for (int i = 0; i < servers.Count; i++)
            {
                Server server = servers[i];
                RadioButton serverrb = GameArea.FindName("s" + i) as RadioButton;
                serverrb.Content = server.Name;
                serverrb.Visibility = Visibility.Visible;
                serverrb.Tag = server;
            }
        }

        private void area_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton rb = (RadioButton)sender;
            LoadServers((List<Server>)rb.Tag);
        }

        private void server_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton rb = (RadioButton)sender;
            Server server = (Server)rb.Tag;
            option.Server = new ServerInfo
            {
                Address = server.Address,
                Port = server.Port
            };
            option.Version = launcher.GetVersion(server.Version);
        }

        private void StartGame_Click(object sender, RoutedEventArgs e)
        {
            Console.WriteLine("StartGame");
            option.Mode = LaunchMode.MCLauncher;
            option.MaxMemory = 2048;
            option.Authenticator = new OfflineAuthenticator(username.Text); // offline
            launcher.JavaPath = SystemTools.FindJava().First();
            launcher.GameLog += launcher_GameLog;
            launcher.Launch(option);
        }

        void launcher_GameLog(LaunchHandle arg1, string log)
        {
            Console.WriteLine(log);
        }
        #endregion
    }
}
